import io
import os
from tqdm import tqdm

import replicate

def transcribe(target, to=None, duplicate="skip", **params):

    model = replicate.models.get("openai/whisper")
    version = model.version("30414ee7c4fffc37e260fcab7842b5be470b9b840f2b608f5baa9bbef9a259ed")

    # Define a dictionary to map the input types to the corresponding functions
    transcribe_functions = {
        "file": transcribe_file,
        "dir": transcribe_dir,
        "file_list": transcribe_files,
        "dir_list": transcribe_dirs
    }

    # Determine the input type of the `target` parameter and call the corresponding function
    if isinstance(target, str) and os.path.isfile(target):
        input_type = "file"
    elif isinstance(target, str) and os.path.isdir(target):
        input_type = "dir"
    elif isinstance(target, list) and all([os.path.isfile(item) for item in target]):
        input_type = "file_list"
    elif isinstance(target, list) and all([os.path.isdir(item) for item in target]):
        input_type = "dir_list"
    else:
        raise ValueError("Invalid input type for `target`")

    return transcribe_functions[input_type](version, target, to=to, duplicate=duplicate, **params)

def transcribe_file(version, filepath, **params):
    # Transcribe the audio
    try:
        result = version.predict(
            audio=open(f'{filepath}' 'rb'), **params)['transcription']
    except Exception as e:
        print("Error: ", e)
        result = None

    # Return the result
    return result

def transcribe_files(version, filepaths, to=None, duplicate_options="skip", **params):
    results = []
    for filepath in tqdm(filepaths, total=len(filepaths), desc="Transcribing files"):
        # Transcribe the file
        result = transcribe_file(version, filepath, **params)
        results.append(write_or_return(filepath, result, to))
    return results

def transcribe_dir(version, folder, to=None, duplicate_options="skip", **params):
    # Get the list of filepaths in the folder
    filepaths = [os.path.join(folder, filename) for filename in os.listdir(folder)]
    return transcribe_files(version, filepaths, to, duplicate_options, **params)

def transcribe_dirs(version, folders, to=None, duplicate_options="skip", **params):
    results = []
    for folder in tqdm(folders, total=len(folders), desc="Transcribing folders"):
        result = transcribe_dir(version, folder, to, duplicate_options, **params)
        results.extend(result)
    return results


# Define a helper function to write the results to a file or return them as a string
def write_or_return(filename, result, folder=None):
    if filename is None:
        raise ValueError("Transcribed filename must be specified")
    elif result is None:
        return None
    elif folder is None:
        return result
    else:
        file_path = os.join(folder, filename)
        with open(file_path, "w") as f:
            f.write(result)
        print(f"Transcribed file saved to {file_path}")


import click
from whisper.main import transcribe

@click.command()
@click.argument("target")
@click.option("--to", "-o", default=None, help="Output directory for transcriptions. Specify None to return the transcriptions.")
@click.option("--duplicate", "-d", default="skip", type=click.Choice(["skip", "rename", "overwrite"]), help="What to do if an output file already exists.")
@click.option("--model", default="base", type=click.Choice(["tiny", "base", "small", "medium", "large-v1", "large-v2"]), help="The model to use for transcription.")
@click.option("--transcription", default="plain text", type=click.Choice(["plain text", "srt", "vtt"]), help="The format for the transcription.")
@click.option("--translate", is_flag=True, help="Translate the text to English when set to True.")
@click.option("--language", default=None, type=str, help="Language spoken in the audio, specify None to perform language detection.")
def cli(target, to, duplicate, model, transcription, translate, language):
    # Map the command-line options to the parameters of the transcribe function
    params = {
        "model": model,
        "transcription_format": transcription,
        "translate": translate,
        "language": language
    }

    # Call the transcribe function with the specified options
    transcribe(target, to=to, duplicate=duplicate, **params)


if __name__ == "__main__":
    cli()



from setuptools import setup, find_packages

setup(
    name='replicate_api_utils',
    version='0.1.0',
    author='Julian Otto',
    author_email="julianotto@outlook.com",
    description='Python package for accessing the Replicate API',
    packages=find_packages(),
    install_requires=[
        'click',
        'replicate',
        # any other necessary dependencies
    ],
    entry_points={
        "console_scripts": [
            "whisper=replicate_api_utils.whisper.cli:main"
        ]
    },
)
